<?php
// Copyright Brian Quade, BQ Plugins 2012, bqplugins.com

if (!defined ('ABSPATH'))
	die ();

require_once 'bq-download-model.php';
require_once 'bq-download-util.php';

class BQPager
{
	const cookie_name = 'bq_download_pager';

	protected static $pager_list = array ();

	protected $name; // name of this pager
	protected $item_count; // number of items in the paged list
	protected $start; // item beginning current page (1-based)
	protected $count; // count per page

	// must be called before headers are sent
	// item_max is the number of items in this pager list
	public function __construct ($pager_name, $item_count)
	{
		$this->name = $pager_name;
		$this->item_count = $item_count;

		$this->read_cookie ();
	}

	// sets the data members and fills the static array if it is still empty
	protected function read_cookie ()
	{
		// if this page is already set in the static list, use those values to initialize this class
		if (isset (self::$pager_list [$this->name]))
		{
			$this->set_values (self::$pager_list [$this->name] ['start'], self::$pager_list [$this->name] ['count']);
			return;
		}

		// read the cookie
		$raw_cookie = BQDownloadUtil::bq_get_cookie (self::cookie_name);
		$cookie_was_empty = $raw_cookie === false;

		if (!$cookie_was_empty)
		{
			// read the cookie into memory
			$pager_settings = explode ('|', $raw_cookie);
			foreach ($pager_settings as $settings)
			{
				list ($name, $start, $count) = explode (',', $settings);
				self::$pager_list [$name] = array ('start' => $start, 'count' => $count);

				if (strcmp ($name, $this->name) == 0)
					$this->set_values ($start, $count, false);
			}

			// if the settings were not found in the cookie, add default ones
			$this_pager_set = isset (self::$pager_list [$this->name]);
			if (!$this_pager_set)
				$this->add_new_pager ();
		}
		else // the cookie was empty
			$this->add_new_pager ();
	}

	// writes the cookie from the static array
	protected function write_cookie ()
	{
		if (headers_sent ())
			return false;

		// construct the cookie string
		$cookie_str = '';
		foreach (self::$pager_list as $name => $settings)
		{
			if (strlen ($cookie_str) > 0)
				$cookie_str .= '|';
			$cookie_str .= $name.','.$settings ['start'].','.$settings ['count'];
		}

		// write the cookie
		BQDownloadUtil::bq_set_cookie (self::cookie_name, $cookie_str);

		return true;
	}

	protected function add_new_pager ()
	{
		$this->set_values (1, $this->item_count > 20 ? 20 : 0);
	}

	protected function set_values ($start, $count, $write = true)
	{
		// some items might have been removed since cookie was last updated
		$this->count = (int) ($count < $this->item_count ? $count : self::get_next_highest_count ($this->item_count));
		$this->start = (int) $this->get_page_start ($start <= $this->item_count ? $this->get_page ($start) : $this->get_total_pages ());

		// update the static array
		self::$pager_list [$this->name] = array ('start' => $this->start, 'count' => $this->count);

		return !$write || $this->write_cookie ();
	}

	public function get_start ()
	{
		return $this->start;
	}

	// returns true if the new settings were saved in the cookie
	public function set_start ($value)
	{
		return $this->set_values ($value, $this->count);
	}

	public function get_count ()
	{
		return $this->count;
	}

	// returns true if the new settings were saved in the cookie
	public function set_count ($value)
	{
		return $this->set_values ($this->start, $value);
	}

	// pass in a page number
	// return the number of items that will be displayed on that page
	public function get_items_on_page ($page)
	{
		if ($this->is_all ())
			return $this->item_count;

		$total_pages = self::get_total_pages ();
		if ($page > $total_pages)
			return 0;

		$extra_items = $this->item_count % $this->count;
		return ($page == $total_pages && $extra_items != 0) ? $extra_items : $this->count;
	}

	// pass in a page number
	// returns the item number that page starts on
	public function get_page_start ($page)
	{
		return (($page - 1) * $this->count) + 1;
	}

	// pass in a page number
	// returns the item number that page ends on
	public function get_page_end ($page)
	{
		$items_on_page = $this->get_items_on_page ($page);
		return (($page - 1) * $this->count) + $items_on_page;
	}

	// pass item number (defaults to current start - 1)
	// returns page number that item is on
	public function get_page ($start = null)
	{
		if ($this->is_all ()) // prevents divide by zero below
			return 1;

		if ($start === null)
			$start = $this->start;

		$page = floor ($start / $this->count);
		if ($start % $this->count != 0)
			++$page;

		return $page;
	}

	// return the total number of pages for this item
	public function get_total_pages ()
	{
		return self::get_page ($this->item_count);
	}

	public function is_all ()
	{
		return $this->count == 0;
	}

	// list is a list of items that have already been sorted
	// start and count determine the page of items to be extracted from the list
	// returns the items from the list that are in the page defined by the current pager settings
	public function manual_limit ($list)
	{
		$this->item_count = count ($list);
		$this->set_values ($this->start, $this->count);

		$page_end = $this->get_page_end ($this->get_page ());

		$new_list = array ();
		for ($e = $this->start - 1; $e < $page_end; $e++)
			$new_list [] = $list [$e];
		return $new_list;
	}

	public function get_limit_clause ()
	{
		return !$this->is_all () ? 'limit '.($this->start - 1).', '.$this->count : '';
	}

	public function get_count_options ()
	{
		$options = array (array ('text' => 20, 'value' => 20, 'selected' => $this->count == 20));
		if ($this->item_count >= 50)
		{
			$options [] = array ('text' => 50, 'value' => 50, 'selected' => $this->count == 50);
			if ($this->item_count >= 100)
			{
				$options [] = array ('text' => 100, 'value' => 100, 'selected' => $this->count == 100);
				if ($this->item_count >= 500)
					$options [] = array ('text' => 500, 'value' => 500, 'selected' => $this->count == 500);
			}
		}
		if ($this->item_count <= 500)
			$options [] = array ('text' => 'All', 'value' => 0, 'selected' => $this->is_all ());

		return $options;
	}

	public function get_page_options ()
	{
		$options = array ();
		$last_page = $this->get_total_pages ();
		for ($p = 1; $p <= $last_page; $p++)
		{
			$s = $this->get_page_start ($p);
			$options [] = array ('page' => $p, 'start' => $s, 'end' => $this->get_page_end ($p), 'selected' => $s == $this->start);
		}

		return $options;
	}

	public static function expire ()
	{
		BQDownloadUtil::bq_delete_cookie (self::cookie_name);
	}

	public static function get_next_highest_count ($count)
	{
		if ($count > 500)
			return 500;
		else if ($count > 100)
			return 100;
		else if ($count > 50)
			return 50;
		else if ($count > 20)
			return 20;

		return 0; // all
	}
}
?>
