<?php
// Copyright Brian Quade, BQ Plugins 2012, bqplugins.com

if (!defined ('ABSPATH'))
	die ();

require_once 'bq-download-model.php';
require_once 'bq-download-util.php';

class BQSorter
{
	const cookie_name = 'bq_download_sorter';

	protected static $sorter_list = array ();

	protected $name; // name of this sorter
	protected $field; // field this sorter is sorted on
	protected $asc; // true is sorting ascending, false if sorting descending
	protected $field_list = array ();

	// must be called before headers are sent
	// item_max is the number of items in this sorter list
	public function __construct ($sorter_name, $sort_fields = null)
	{
		$this->name = $sorter_name;

		if ($sort_fields !== null)
			$this->field_list = $sort_fields;
		else
		{
			// known sorter types
			switch ($sorter_name)
			{
				case 'file':
					$this->field_list = array ('name', 'size', 'last_uploaded', 'last_downloaded', 'download_count');
					break;
				case 'email':
					$this->field_list = array ('email', 'request_count', 'download_count');
					break;
			}
		}

		$this->read_cookie ();
	}

	// sets the data members and fills the static array if it is still empty
	protected function read_cookie ()
	{
		// if this page is already set in the static list, use those values to initialize this class
		if (isset (self::$sorter_list [$this->name]))
		{
			$this->field = self::$sorter_list [$this->name] ['field'];
			$this->asc = self::$sorter_list [$this->name] ['asc'] != 0;
			return;
		}

		// read the cookie
		$raw_cookie = BQDownloadUtil::bq_get_cookie (self::cookie_name);
		$cookie_was_empty = $raw_cookie === false;

		if (!$cookie_was_empty)
		{
			// read the cookie into memory
			$sorter_settings = explode ('|', $raw_cookie);
			foreach ($sorter_settings as $settings)
			{
				list ($name, $field, $asc) = explode (',', $settings);
				$asc = $asc == 1;
				self::$sorter_list [$name] = array ('field' => $field, 'asc' => $asc);

				if (strcmp ($name, $this->name) == 0)
					$this->set_values ($field, $asc, false);
			}

			// if the settings were not found in the cookie, add default ones
			$this_sorter_set = isset (self::$sorter_list [$this->name]);
			if (!$this_sorter_set)
				$this->add_new_sorter ();
		}
		else // the cookie was empty
			$this->add_new_sorter ();
	}

	// writes the cookie from the static array
	protected function write_cookie ()
	{
		if (headers_sent ())
			return false;

		// construct the cookie string
		$cookie_str = '';
		foreach (self::$sorter_list as $name => $settings)
		{
			if (strlen ($cookie_str) > 0)
				$cookie_str .= '|';
			$cookie_str .= $name.','.$settings ['field'].','.($settings ['asc'] ? 1 : 0);
		}

		// write the cookie
		BQDownloadUtil::bq_set_cookie (self::cookie_name, $cookie_str);

		return true;
	}

	protected function add_new_sorter ()
	{
		if (count ($this->field_list) == 0)
			return false;

		$this->set_values ($this->field_list [0], true);
		return true;
	}

	protected function set_values ($field, $asc, $write = true)
	{
		// some items might have been removed since cookie was last updated
		$this->field = $field;
		$this->asc = $asc != 0;

		// update the static array
		self::$sorter_list [$this->name] = array ('field' => $this->field, 'asc' => $this->asc);

		return !$write || $this->write_cookie ();
	}

	public function get_field ()
	{
		return $this->field;
	}

	// returns true if the new settings were saved in the cookie
	public function set_field ($value)
	{
		return $this->set_values ($value, $this->asc);
	}

	public function get_asc ()
	{
		return $this->asc;
	}

	// returns true if the new settings were saved in the cookie
	public function set_asc ($value)
	{
		return $this->set_values ($this->field, $value);
	}

	public function get_order_by_clause ()
	{
		// double check security of text cookie
		global $wpdb;
		if (strcmp ($wpdb->_escape ($this->field), $this->field) != 0)
			return '';

		$sort_dir = $this->asc ? 'asc' : 'desc';
		return 'order by '.$this->field.' '.$sort_dir;
	}

	public function display_control ($visible, $field_name = null, $asc_label = null, $desc_label = null, $type = null)
	{
		$sort = $this->get_field ();
		$sort_dir = $this->get_asc () ? 'asc' : 'desc';
		$sort_type = $type;
		$invisible = !$visible;
		$sort_field = $field_name;
		$asc_title = $asc_label;
		$desc_title = $desc_label;
		ob_start ();
		include 'bq-download-sort-control.html.php';
		return ob_get_clean ();
	}

	public function manual_sort ($items)
	{
		// can only do manually sorts on known fields in known sorter types
		switch ($this->name)
		{
			case 'file':
				switch ($this->field)
				{
					case 'size':
						usort ($items, array ($this, 'compare_file_sizes'));
						break;
					case 'last_uploaded':
						usort ($items, array ($this, 'compare_file_dates'));
						break;
				}
				break;
//			case 'email':
//				break;
		}

		return $items;
	}

	protected function compare_file_sizes ($a, $b)
	{
		if ($this->asc)
			return ($a->size < $b->size) ? -1 : 1;
		else
			return ($a->size > $b->size) ? -1 : 1;
	}

	protected function compare_file_dates ($a, $b)
	{
		list ($date_a, $time_a) = explode (' ', $a->last_uploaded);
		list ($year_a, $month_a, $day_a) = explode ('-', $date_a);
		list ($hour_a, $minute_a, $second_a) = explode (':', $time_a);
		$time_a = mktime ($hour_a, $minute_a, $second_a, $month_a, $day_a, $year_a);

		list ($date_b, $time_b) = explode (' ', $b->last_uploaded);
		list ($year_b, $month_b, $day_b) = explode ('-', $date_b);
		list ($hour_b, $minute_b, $second_b) = explode (':', $time_b);
		$time_b = mktime ($hour_b, $minute_b, $second_b, $month_b, $day_b, $year_b);

		if ($this->asc)
			return ($time_a < $time_b) ? -1 : 1;
		else
			return ($time_a > $time_b) ? -1 : 1;
	}

	public static function expire ()
	{
		BQDownloadUtil::bq_delete_cookie (self::cookie_name);
	}
}
?>
